;*********************************************************************
; Liest Spannung 0,00V...29,5V mit Auto-Range
; Out   : Wert in Anzeigeregister  DIGIT_1...4
; Scratch-Reg: TEMP_A, TEMP_B
;*********************************************************************
get_spannung:
		rcall	get_adc_64			; ADC lesen, 64 Werte
;
		sbis	BER_DDR,BER_30V		; +30V
		rjmp	get_spannung_10v	; nein, 10V testen
;
		load_p	Z,99				; Test, ob auf 10V geschaltet werden mu
		sub		ZL,TEMP_A
		sbc		ZH,TEMP_B			; ja im 30V Bereich, kleiner als 9,9V?
		brcs	get_spannung_ok		; nein, ok
;
		rcall	set_10v				; kleiner, umschalten und neu messen
		rjmp	get_spannung_neu			
;
get_spannung_10v:
		load_p	Z,1000				; im 10V Bereich, grer als 10V?
		sub		ZL,TEMP_A
		sbc		ZH,TEMP_B
		brcc	get_spannung_ok
;
		rcall	set_30v				; grer, umschalten und neu messen
;
get_spannung_neu:
		rcall	get_adc_64
;
get_spannung_ok:
		sbis	BER_DDR,BER_30V		; 30V an?
		rjmp	get_spannung_range_ok; nein
;
		load_p	X,295				; 29,5V
		sub		XL,TEMP_A
		sbc		XH,TEMP_B
		brcc	get_spannung_range_ok
;
		rcall	Error				; 'E' = Error ausgeben
		rjmp	get_spannung_end
;
get_spannung_range_ok:
		rcall	show_16bit_wert
;
get_spannung_end:
		ret
;
;*********************************************************************
; Liest Logikpegel
; Out   : XH, XL
; Scratch-Reg: -
;*********************************************************************
get_logik:
		sbic	LOGIK_PORT,LOGIK_PIN			; L? 1M ist an GND
		rjmp	get_logik_signal_ok				; H, Signal liegt an
;
		sbi		BER_PORT,BER_R_IN				; 1M an +Ub
		rcall	wait_5ms
		sbis	LOGIK_PORT,LOGIK_PIN			; H?
		rjmp	get_logik_signal_ok				; L, Signal liegt an
;
		ldi		TEMP_A,0b10000000				; Eingang offen: '-'
		rjmp	get_logik_end					; ok

get_logik_signal_ok:
		ldi		TEMP_A,(1<<ICF1)
		out		TIFR,TEMP_A						; IRQ vorsichtshalber Flag lschen
;
		rcall	wait_5ms
		in		TEMP_A,TIFR
		sbrc	TEMP_A,ICF1						; Pulse?
		rjmp	get_logik_pulse					; ja
;
; falls zu hohe Frequenz mal mit dem Counter probieren...				
		ldi		TEMP_A,(1<<CS11)|(1<<CS12)		; Clock extern, fallende Flanke
		out		TCCR1B,TEMP_A					; Counter starten, extern
		rcall	wait_5ms
;
		in		TEMP_A,TCNT1L					; Counterwert holen		
		in		TEMP_B,TCNT1H		
;
		out		TCCR1B,NULL						; Counter Stop
        out     TCNT1H,NULL						; Counter auf 0		
        out     TCNT1L,NULL		
;		
		tst		TEMP_B							; Pulse?
		brne	get_logik_pulse					; ja
		cpi		TEMP_A,2						; Umschaltpulse Widerstnde ausblenden
		brlo	get_logik_no_pulse				; nein
;
get_logik_pulse:
		ldi		TEMP_A,0b11010011				; 'P'
		rjmp	get_logik_end
;
get_logik_no_pulse:
		sbi		BER_DDR,BER_RC					; 10V-Bereich ein
		rcall	wait_5ms
		mov		TEMP_A,ADC_WERT_L
		mov		TEMP_B,ADC_WERT_H
;
		lds		XL,L_WERT
		lds		XH,L_WERT+1
		sub		XL,TEMP_A
		sbc		XH,TEMP_B
		brcc	get_logik_l
;
get_logik_l_h:
		lds		XL,H_WERT
		lds		XH,H_WERT+1
		sub		XL,TEMP_A
		sbc		XH,TEMP_B
		brcs	get_logik_h
;
		ldi		TEMP_A,0b00001001				; verbotener Bereich '='
		rjmp	get_logik_end
;		
get_logik_l:
		ldi		TEMP_A,0b01011000				; Low: 'L'
		rjmp	get_logik_end
;
get_logik_h:
		ldi		TEMP_A,0b11010110				; High: 'H'
;
get_logik_end:		
		cbi		BER_DDR,BER_RC					; 10V wieder hochohmig
		cbi		BER_PORT,BER_R_IN				; 1M wieder an GND
		rcall	wait_5ms
		sts		DIGIT_4,TEMP_A					; Symbol von oben auf Digit schreiben
		ret
;
;*********************************************************************
; Schreibt CL = Clock auf Display
; Out   : XH, XL
; Scratch-Reg: -
;*********************************************************************
get_clock:
		ldi		TEMP_A,0b01011001				; 'C'
		sts		DIGIT_4,TEMP_A
		ldi		TEMP_A,0b01011000				; 'L'
		sts		DIGIT_3,TEMP_A
		ret
;
;*********************************************************************
; Misst Frequenz 0,001kHz bis ca 22MHz mit Auto-Range
; Out:  Wert in Anzeigeregister DIGIT_1...4  
; Scratch-Reg: TEMP_A, TEMP_B, TEMP_C, XH, XL
;*********************************************************************
get_frequenz:									; TEMP_C = Digit mit Dezimalpunkten
		load_p	X,2								; 2ms Torzeit, max Frequenz ~22MHz
		sbr		FLAGS_IRQ,(1<<FLAG_FREQ)		; Messung im nchsten IRQ starten
;
get_frequenz_loop_0:
		sbrc	FLAGS_IRQ,FLAG_FREQ				; Messung Ende
		rjmp	get_frequenz_loop_0
;
; Wert = Messfrequenz * Torzeit / Prescaler (4), z.B. bei 10MHz => Wert = 5000
		ldi		TEMP_C, 5						; Ergebnis durch 5 teilen
		mov		DIV_L, TEMP_C					; Nenner = 5
		clr		DIV_M
		clr		DIV_H
		mov		TEMP_A, CNT_WERT_L				; Zhler = Messwert
		mov		TEMP_B, CNT_WERT_H
		clr		TEMP_C
		rcall	div24u							; Ergebnis zurck in TEMP_C..TEMP_A
;
		mov		CNT_WERT_L, TEMP_A
		mov		CNT_WERT_H, TEMP_B
;
		ldi		TEMP_C, 0b00010100				; xx.xx.MHz 2 Dezimalpunkte: Digits 1 & 3
;
		load_p	X,999							; richtiger Messbereich >3 Digits?
		sub		XL,CNT_WERT_L
		sbc		XH,CNT_WERT_H
		brcs	get_frequenz_ok					; wenn ja, fertig
;
		load_p	X,4								; sonst kleineren Bereich => 4ms
		clr		TEMP_C							; 9999kHz, hier kein Dezimalpunkt
		sbr		FLAGS_IRQ,(1<<FLAG_FREQ)		; Messung im nchsten IRQ starten
;
get_frequenz_loop_1:
		sbrc	FLAGS_IRQ,FLAG_FREQ				; Messung Ende
		rjmp	get_frequenz_loop_1
;
		load_p	X,999							; richtiger Messbereich >3 Digits?
		sub		XL,CNT_WERT_L
		sbc		XH,CNT_WERT_H
		brcs	get_frequenz_ok					; wenn ja, fertig
;		
		load_p	X,40							; sonst kleineren Bereich => 40ms
		ldi		TEMP_C,(1<<DIG_2)				; 999,9 kHz, Dezimalpunkt = Digit2
		sbr		FLAGS_IRQ,(1<<FLAG_FREQ)		; Messung im nchsten IRQ starten
;
get_frequenz_loop_2:
		sbrc	FLAGS_IRQ,FLAG_FREQ				; Messung Ende
		rjmp	get_frequenz_loop_2
;
		load_p	X,999							; richtiger Messbereich >3 Digits?
		sub		XL,CNT_WERT_L
		sbc		XH,CNT_WERT_H
		brcs	get_frequenz_ok					; wenn ja, fertig
;			
		load_p	X,400							; sonst kleineren Bereich => 400ms
		ldi		TEMP_C,(1<<DIG_3)				; 99,99 kHz, Dezimalpunkt = Digit3
		sbr		FLAGS_IRQ,(1<<FLAG_FREQ)		; Messung im nchsten IRQ starten
;
get_frequenz_loop_3:
		sbrc	FLAGS_IRQ,FLAG_FREQ				; Messung Ende
		rjmp	get_frequenz_loop_3
;
		load_p	X,199							; ab 2kHz Torzeit basierte Frequenzmessung
		sub		XL,CNT_WERT_L					; da dann stabiler als Periodenmessung
		sbc		XH,CNT_WERT_H
		brcs	get_frequenz_ok					; wenn ja, fertig
;
		rcall	get_periode						; unterhalb von 2kHz Periodendauer messen
		rjmp	get_frequenz_periode_ok
;
get_frequenz_ok:
		mov		TEMP_A,CNT_WERT_L
		mov		TEMP_B,CNT_WERT_H
;
		load_p	X,9999							; berlauf?
		sub		XL,TEMP_A
		sbc		XH,TEMP_B
		brcs	get_frequenz_error
;
get_frequenz_periode_ok:
		mov		DIG_DP,TEMP_C
		rcall	show_16bit_wert
		rjmp	get_frequenz_end
;
get_frequenz_error:
		rcall	Error							; 'E' = Error ausgeben
;
get_frequenz_end:
		out		TCCR1B,NULL						; Counter stop
		cbr		FLAGS,(1<<FLAG_IRQ)				; Flag lschen
		ret
;
Error:	ldi		TEMP_A,0b11011001				; Segmentmuster: 'E' = Error
		sts		DIGIT_1,TEMP_A
		sts		DIGIT_2,NULL
		sts		DIGIT_3,NULL
		sts		DIGIT_4,NULL
		ret
;
;************************************************************************************
; Misst Frequenz von 1Hz...1999Hz ber Periodendauer mit Counter 1 (24 Bit Werte)
; Es wird durchgngig mit 16MHz Zhltakt gearbeitet, der Prescaler:8 entfllt dann.
;************************************************************************************
get_periode:
		cbi		BER_DDR,BER_RC					; RC hochohmig schalten
;
		clr		cnt_wert_l						; Ziel lschen wegen mehrerer Messungen
		clr		cnt_wert_h
		clr		r28								; noch freies Register fr 3. Byte
		ldi		TEMP_D, 16						; 16 Durchlufe fr Mittelwertbildung
;
get_periode_messen:
		cli
		ldi		TEMP_A,(1<<ICF1)
		out		TIFR,TEMP_A						; T1 Int Capture Flag lschen
		in		TEMP_A,TIMSK
		sbr		TEMP_A,(1<<TICIE1) | (1<<TOIE1)	; T1 Capture- & Overflow einschalten
		out		TIMSK,TEMP_A
		sei
;
		sbr		FLAGS_IRQ,(1<<FLAG_PERIODE_1F)	; auf 1. Flanke warten
		sbr		FLAGS_IRQ,(1<<FLAG_PERIODE)		; Messung starten
;
		load_p	X,0								; Timeout setzen
get_periode_loop:
		subi	r29, 128						; Bit 7 toggeln um Timeout zu verdoppeln
		sbrs	r29, 7							; Untergrenze dann ca. 33Hz (r29 noch frei)
		sbiw	X,1								; Subtract immediate from Word
		breq	get_periode_error				; Timeout, wenn 0
		sbrc	FLAGS_IRQ,FLAG_PERIODE			; Messung fertig?				
		rjmp	get_periode_loop				; nein
;
		add		cnt_wert_l, DIV_L
		adc		cnt_wert_h, DIV_M
		adc		r28, DIV_H
;
		dec		TEMP_D							; Anzahl Messungen erreicht?
		brne	get_periode_messen				; nein
;
		lsr		r28								; /2
		ror		cnt_wert_h
		ror		cnt_wert_l
		lsr		r28								; /4
		ror		cnt_wert_h
		ror		cnt_wert_l
		lsr		r28								; /8
		ror		cnt_wert_h
		ror		cnt_wert_l
		lsr		r28								; /16
		ror		cnt_wert_h
		ror		cnt_wert_l
;
		mov		DIV_H, r28						; Mittelwert zurck in Divisor kopieren
		mov		DIV_M, cnt_wert_h
		mov		DIV_L, cnt_wert_l
;
		cli
		in		TEMP_A,TIMSK
		cbr		TEMP_A,(1<<TICIE1) | (1<<TOIE1)	; T1 Capture- & Overflow ausschalten
		out		TIMSK,TEMP_A
		sei
;
;		f = F_CPU / Zhlerstand (DIV_H, DIV_M, DIV_L)
		ldi		TEMP_A,low(F_CPU)				; Zhler fr 24/24 Bit Division
		ldi		TEMP_B,high(F_CPU)				; Nenner aus ICP-Capture in DIV_H .. DIV_L
		ldi		TEMP_C,byte3(F_CPU)
		rcall	div24u							; Ergebnis zurck in TEMP_C .. TEMP_A
		ldi		TEMP_C,(1<<DIG_4)				; Dezimalpunkt: 9,999kHz
		rjmp	get_periode_end
;
get_periode_error:								; Timeout: kein Signal oder f zu niedrig
		cbr		FLAGS_IRQ,(1<<FLAG_PERIODE)
		clr		TEMP_A							; Display auf 0
		clr		TEMP_B
		clr		TEMP_C							; DP aus
;
get_periode_end:
		sbi		BER_DDR,BER_RC					; RC auf GND
		out		TCCR1B,NULL						; Counter stop
    	out		TCNT1H,NULL						; Counter auf 0		
   		out		TCNT1L,NULL
		ret
