;************** IRQ - Routinen ********************************************				 

;**************************************************************************
; 1ms Timer 2 Interrupt => Zeitbasis fr die Messungen & Display-Multiplex
; Scratch-Reg: -
;**************************************************************************
irq_1ms:in		SREG_SAVE,SREG
		push	SREG_SAVE
		push	TEMP_A
		push	TEMP_B
		push	ZH
		push	ZL
;
		sbrs	FLAGS,FLAG_C				; Clock Out?
		rjmp	irq_1ms_no_clock			; nein
;
		sei									; IRQ wegen Overflow Timer 0 freigeben
		rjmp	irq_1ms_no_frequenz
;
irq_1ms_no_clock:
		sbrc	FLAGS_IRQ,FLAG_FREQ			; Frequenzmessung?
		rjmp	irq_1ms_frequenz			; ja
;
		sbrs	FLAGS_IRQ,FLAG_PERIODE		; Periodendauermessung?
		rjmp	irq_1ms_no_frequenz			; nein
;
		sei									; IRQ wegen Capture Timer 1 wieder freigeben
		rjmp	irq_1ms_display
;
irq_1ms_frequenz:
		ldi		TEMP_A,(1<<CS11)|(1<<CS12)	; Clock extern, fallende Flanke
		out		TCCR1B,TEMP_A				; Counter starten, extern
		sbiw	X,1							; Anzahl Durchlufe
		brcc	irq_1ms_no_frequenz			; nicht fertig
;
		in		CNT_WERT_L,TCNT1L			; Counterwert holen
		in		CNT_WERT_H,TCNT1H
;
		cbr		FLAGS_IRQ,(1<<FLAG_FREQ)	; Flag lschen, fertig
;
		out		TCCR1B,NULL
        out     TCNT1H,NULL					; Counter auf 0
        out     TCNT1L,NULL
;
irq_1ms_no_frequenz:
		in		ADC_WERT_L,ADCL				; immer ADC-Wert holen
		in		ADC_WERT_H,ADCH
;
; ---> Tastenabfrage
		sbrc	FLAGS_IRQ,FLAG_KEY			; letzter Tastendruck schon bearbeitet?
		rjmp	irq_1ms_key_end				; nein
;
		in		TEMP_A,KEY_PORT				; Key-Status holen
		andi	TEMP_A,(1<<KEY_PIN)			; ausmaskieren
		cp		TEMP_A,KEY_TEMP				; gleicher Zustand?
		breq	irq_1ms_key_inc				; ja
;
		mov		KEY_TEMP,TEMP_A				; nein. merken
		clr		KEY_CNT						; Zhler auf 0
		rjmp	irq_1ms_key_end
;
irq_1ms_key_inc:
		inc		KEY_CNT						; Zhler erhhen
		cpi		KEY_CNT,KEY_WAIT			; Anzahl IRQs erreicht?
		brlo	irq_1ms_key_end				; nein
;
		clr		KEY_CNT						; ja, Zhler lschen
		sbrs	KEY_TEMP,KEY_PIN			; Taste ist losgelassen
		rjmp	irq_1ms_key_press			; nein, gedrckt
;
		cbr		FLAGS_IRQ,(1<<FLAG_KEY_PRESS)	; gedrckt-Flag lschen
		rjmp	irq_1ms_key_end
;
irq_1ms_key_press:
		sbrs	FLAGS_IRQ,FLAG_KEY_PRESS	; war noch nicht losgelassen
		sbr		FLAGS_IRQ,(1<<FLAG_KEY)|(1<<FLAG_KEY_PRESS)		; Taste gltig
;
irq_1ms_key_end:
		sbr		FLAGS,(1<<FLAG_IRQ)			; Flag fr neuen Wert
;
; ---> Display-Multiplex
irq_1ms_display:
		in		TEMP_A,DIG_PORT				; Digit-Zustand holen
		andi	TEMP_A,~((1<<DIG_1)|(1<<DIG_2)|(1<<DIG_3)|(1<<DIG_4)|(1<<SEG_DP))
		out		DIG_PORT,TEMP_A				; Digits und DP ausschalten
;
		ldi		ZH,high(DIGIT_1)			; erstes Digit -> H-Byte
		mov		ZL,DIG_ADR					; Digit-Nummer -> L-Byte (Offset)
		ld		TEMP_B,Z					; Segment-Muster holen
;
		out		SEG_PORT,TEMP_B				; und setzen
;
		or		TEMP_A,DIG_NR				; neue Digit-Nummer einschalten
		out		DIG_PORT,TEMP_A				; und setzen
;
		mov		TEMP_A, DIG_NR				; Dezimalpunkt fr aktuelles Digit?
		and		TEMP_A, DIG_DP				; steht eine 1 an gleicher Stelle?
		breq	irq_1ms_no_dp				; Sprung wenn nicht
;
		sbi		DIG_PORT,SEG_DP				; sonst Dezimalpunkt einschalten
;
irq_1ms_no_dp:
		inc		DIG_ADR						; nchste Adresse
		lsl		DIG_NR						; nchste Stelle, 0 wenn Ende
		brne	irq_1ms_end
;
		ldi		DIG_ADR,low(DIGIT_1)		; wieder auf Anfang
		ldi		DIG_NR,(1<<DIG_1)
;
irq_1ms_end:
		pop		ZL			
		pop		ZH
		pop		TEMP_B
		pop		TEMP_A
		pop		SREG_SAVE
		out		SREG,SREG_SAVE
		reti
;
;********************************************************************
; Overflow Timer1 Interrupt: Inkrementiert ofl_T1 bei Periodendauermessung
;********************************************************************
TIM1_OVF:
		in		SREG_SAVE,SREG
		push	SREG_SAVE
		inc		Ofl_T1
		pop		SREG_SAVE
		out		SREG,SREG_SAVE
		reti
;
;*********************************************************************
; Timer1 Capture Interrupt: Kopiert TCNT1H/L bei steigender Signal-
; flanke am ICP-Pin (PB0) nach ICR1H/L fr Periodendauer-Messung
;*********************************************************************
TIM1_ICP:
		in		SREG_SAVE,SREG
		push	SREG_SAVE
		push	TEMP_A
;
		sbrc	FLAGS_IRQ, FLAG_PERIODE_1F	; 1. oder 2. Flanke?
		rjmp	FLANKE1
;
;		2. Flanke: Zhler auslesen
		in		DIV_L, ICR1L				; Capturewert holen (24 Bit)
		in		DIV_M, ICR1H
		mov		DIV_H, Ofl_T1				; Byte 3 aus T1-Overflow dazu
		cbr		FLAGS_IRQ,(1<<FLAG_PERIODE)	; Flag lschen, Messung fertig
		rjmp	FERTIG
;
FLANKE1:; Counter auf 0 => ab jetzt bis zur 2. Flanke die Prozessortakte zhlen
		clr		Ofl_T1		
		out     TCNT1H,NULL
        out     TCNT1L,NULL
		ldi		TEMP_A,(1<<CS10)|(1<<ICNC1)	; Timer1: F_CPU-Takte ohne Prescaler
		out		TCCR1B,TEMP_A				; zhlen, Noise canceler aktivieren
		cbr		FLAGS_IRQ,(1<<FLAG_PERIODE_1F)	
;
FERTIG:	pop		TEMP_A
		pop		SREG_SAVE
		out		SREG,SREG_SAVE
		reti
;
;*********************************************************************
; Overflow Timer0 Interrupt: Erzeugt konstante Frequenz an PC1
; Scratch-Reg: -
;*********************************************************************
TIM0_OVF:										; bei 16MHz Quarz
		in		SREG_SAVE,SREG					; und Prescaler = 8
		push	SREG_SAVE
		push	TEMP_A							; ca 100kHz Output
		push	TEMP_B
;
        ldi     TEMP_A,246						; 256-(16MHz * 10us/2) / 8 = 246
        out     TCNT0,TEMP_A					; Timer-Wert wieder setzen
;
		ldi		TEMP_B,(1<<CLK_OUT)
		in		TEMP_A,BER_PORT					; Zustand Clock-Pin PC1 holen
		eor		TEMP_A,TEMP_B					; Bit toggeln
		out		BER_PORT,TEMP_A					; und wieder ausgeben
;
		pop		TEMP_B
		pop		TEMP_A
		pop		SREG_SAVE
		out		SREG,SREG_SAVE
		reti
