; ATMega8 Mess-Stick / Multimeter
; 10.05.2007 M'Soft Berlin (VER = 0, REV = 5)
;
; 30.09.2015 Thomas N. DIESES UPDATE BETRIFFT DIE URSPRNGLICHE HARDWARE VON MICHAEL U.
;					   ================================================================
;	- Neu erstelle Periodendauer Messung mit ICP-Capture Interrupt und Mittelwert-Bildung ber 16 Messungen
;	  Der uC-Takt wird direkt per 24 Bit Zhler erfasst; der 1:8 Prescaler entfllt daher.
;	  Die kleinste messbare Frequenz liegt hier bei 33Hz (kann man verringern, wenn man das Timeout vergrert)
;	- Zustzlicher Frequenzmessbereich mit 2ms Zyklus bis ca 22MHz, Anzeige dann mit 2 Dezimalpunkten
;	- Neuer Binr zu 7-Segment Dekoder, Divisionsroutine 24/24 Bit statt 32/16 Bit
;
.include "m8def.inc"
.include "Definitionen.inc"
;
;****************************** Vektor-Liste *************************
.CSEG
					rjmp    Reset
.ORG	OC2addr		rjmp	irq_1ms				; Timer/Counter2 Compare Match (Zeitbasis)
.ORG 	OVF1addr 	rjmp	TIM1_OVF			; Timer1 Overflow Handler (16 Bit Zhler)
.ORG	ICP1addr	rjmp	TIM1_ICP			; Timer/Counter1 Capture Event
.ORG	OVF0addr	rjmp	TIM0_OVF			; Timer/Counter0 Overflow (Clock-Out)
;
;*********************************************************************
; Initialisierungsroutine
;*********************************************************************
.ORG	INT_VECTORS_SIZE
Reset:	ldi     TEMP_A,low(RAMEND)				; Hardware initialisieren
		out     SPL,TEMP_A						; Stack an das interne RAM-Ende
		ldi     TEMP_A,high(RAMEND)
		out     SPH,TEMP_A
		clr		NULL							; wird sogleich bentigt!
;
; Port B *************************************************************
; Digit-Treiber: PB2-PB5 = DIG_1-4, PB1=DP (Segment), PB0_ICP Perioden-Messung
		out     PORTB,NULL						; alle Digits aus
		ldi     TEMP_A,(1<<DIG_1)|(1<<DIG_2)|(1<<DIG_3)|(1<<DIG_4)|(1<<SEG_DP)
		out     DDRB,TEMP_A						; Digits Ausgang
;
; Port C *************************************************************
; PC1=Clock out, PC2=Mode-Taste, PC4,5= DVM-Range-Widerstnde, PC0 = AIN/DVM, PC3=AIN/Logik
		ldi		TEMP_A,(1<<KEY_PIN)				; PullUp fr Taste ein, Ausgnge L	
		out     PORTC,TEMP_A 
		ldi		TEMP_A,(1<<CLK_OUT)|(1<<BER_RC)|(1<<BER_R_IN)	; RC an GND, R_IN an GND
		out     DDRC,TEMP_A
;
; Port D *************************************************************
; Segment-Treiber: PD0-PD4 = A-E, PD6,7 = F,G, PD5=T1=Eingang Frequenzmessung
		out     PORTD,NULL                		; Alle Segmente aus
		ldi     TEMP_A,(1<<SEG_A)|(1<<SEG_B)|(1<<SEG_C)|(1<<SEG_D)|(1<<SEG_E)|(1<<SEG_F)|(1<<SEG_G)
		out     DDRD,TEMP_A						; Segmente Ausgang
;
; TIMER 2 - 1ms (8Bit) ***********************************************
; Vorteiler 64, Timer 249 => liefert die Zeitbasis fr Frequenzmessung
		ldi		TEMP_A,(1<<CS22)|(1<<WGM21)	    ; Prescale 64, CTC
		out		TCCR2,TEMP_A
		ldi     TEMP_A,TCONST_TIME				; Timer-Wert
		out     OCR2,TEMP_A		
		out		ASSR,NULL
		ldi		TEMP_A,(1<<OCIE2)				; Timer-IRQ frei
		out     TIMSK,TEMP_A
;
; TIMER 1 - Counter (16Bit) ******************************************
; T1 = Frequenzzhler-Eingang, hier ohne OVF Interrupt, max Count = 9999
		out		TCCR1A,NULL						; Counter stop
		out		TCCR1B,NULL
		out     TCNT1H,NULL			; Counter auf 0	-> Grundstellung in allen
		out     TCNT1L,NULL			; Funktionen am Ende wiederherstellen!
;
; TIMER 0 - extern Clock (8Bit) **************************************
; erzeugt konstante Frequenz auf Ausgang PC1
		ldi		TEMP_A,(1<<CS01)			    ; Prescaler 8-fach
		out		TCCR0,TEMP_A					; Preload in IRQ.INC
;
; ADC ****************************************************************
		ldi		TEMP_A,(1<<REFS0)|(1<<REFS1)	; interne Referenz
		out		ADMUX,TEMP_A
		ldi		TEMP_A,(1<<ADEN)|(1<<ADSC)|(1<<ADFR)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0)
		out		ADCSR,TEMP_A					; Free Running und starten
;
;*********************************************************************		 
; Software initialisieren
		ldi		FLAGS,(1<<FLAG_U)				; Start mit Funktion Spannungsmesser
		clr		FLAGS_IRQ
		clr		DIG_DP							; alle Dezimalpunkte aus
		ldi		DIG_NR,(1<<DIG_1)				; Bit fr aktuelle Digitposition Digit 1
		ldi		DIG_ADR,low(DIGIT_1)			; Adresse Daten fr Digit 1
;
		sts		DIGIT_1,NULL					; Alle Segmente aus
		sts		DIGIT_2,NULL
		sts		DIGIT_3,NULL
		sts		DIGIT_4,NULL
;
		sei
;
		rcall	init							; Mit Ub und setzt Spannungs- und Logikparameter
;
;*********************************************************************
; Main-Loop
;*********************************************************************
main:	sbrs	FLAGS,FLAG_U			; Spannung messen?
		rjmp	main_freq				; nein
		rcall	set_10v
;
main_span:
		rcall	get_spannung
		ldi		TEMP_D,100				; Warteschleife, 100ms -> 10 Messungen/s
		rjmp	main_wait_loop
;
main_freq:
		sbrs	FLAGS,FLAG_F			; Frequenz messen?
		rjmp	main_logik				; nein
		rcall	get_frequenz
;
		ldi		TEMP_D,100				; Warteschleife, 100ms -> max 10 Messungen/s
		rjmp	main_wait_loop
;
main_logik:
		sbrs	FLAGS,FLAG_L			; Logikpegel?
		rjmp	main_clock				; nein
		rcall	get_logik
		sbrs	FLAGS_IRQ,FLAG_KEY		; Taste gedrckt?
		rjmp	main_logik				; nein
		rjmp	main_wait_key
;
main_clock:
		rcall	get_clock
		sbrs	FLAGS_IRQ,FLAG_KEY		; Taste gedrckt?
		rjmp	main_clock				; nein
		rjmp	main_wait_key
;
main_wait_loop:
		sbrs	FLAGS,FLAG_IRQ			; auf IRQ warten
		rjmp	main_wait_loop
;
		cbr		FLAGS,(1<<FLAG_IRQ)		; Flag lschen fr nchsten IRQ
;
		sbrs	FLAGS_IRQ,FLAG_KEY		; Taste gedrckt?
		rjmp	main_wait_no_key		; nein
;
main_wait_key:
		rcall	key_decode				; Taste auswerten, wechselt Funktion U->F->L->C->U usw.
		rjmp	main_key_end
;
main_wait_no_key:
		dec		TEMP_D					; Warteschleife bei U und F
		brne	main_wait_loop
;
main_key_end:
		sbrc	FLAGS,FLAG_U			; Spannung messen
		rjmp	main_span
;
		sbrc	FLAGS,FLAG_F			; Frequenz messen
		rjmp	main_freq
;
		sbrc	FLAGS,FLAG_L			; Logik messen
		rjmp	main_logik
;
		sbrc	FLAGS,FLAG_C			; Clock ausgeben
		rjmp	main_clock
;
		rjmp	main					; hier sollten wir nie ankommen... ;)
;
;*********************************************************************				 
;
.include "Funktionen.inc"
.include "Subroutinen.inc"
.include "Irq.inc"
