;**************************** Allgemeine Routinen **************************

;*********************************************************************
; Key Decode
; wechselt durch alle Funktionen U -> F -> L -> U usw.
; Scratch-Reg: -
;*********************************************************************
key_decode:
		push	TEMP_A
		push	TEMP_B
;
		cbi		BER_DDR,BER_30V			; 30V hochohmig schalten
		cbi		BER_DDR,BER_RC			; RC hochohmig schalten
;
		sbrs	FLAGS,FLAG_U			; Spannung?
		rjmp	key_decode_no_u			; nein
;
; Spannung aus, Frequenz ein		
		cbr		FLAGS,(1<<FLAG_U)		; Spannung aus
		sbr		FLAGS,(1<<FLAG_F)		; Frequenz ein
		rjmp	key_decode_end
;
key_decode_no_u:
		sbrs	FLAGS,FLAG_F			; Frequenz?
		rjmp	key_decode_no_f			; nein
;
; Frequenz aus, Logik ein		
		cbr		FLAGS,(1<<FLAG_F)		; Frequenz aus
		sbr		FLAGS,(1<<FLAG_L)		; Logik ein
		sts		DIGIT_3,NULL
		sts		DIGIT_2,NULL
		sts		DIGIT_1,NULL
		clr		DIG_DP
		rjmp	key_decode_end
;
key_decode_no_f:
		sbrs	FLAGS,FLAG_L			; Logik?
		rjmp	key_decode_no_l			; nein
;
; Logik aus, Clock ein
		cbr		FLAGS,(1<<FLAG_L)		; Logik aus
		sbr		FLAGS,(1<<FLAG_C)		; Clock ein
		sbi		BER_DDR,BER_RC			; RC auf GND
		clr		DIG_DP
		cli
		in		TEMP_A,TIMSK
		sbr		TEMP_A,(1<<TOIE0)		; Timer 0 Overflow-IRQ einschalten
		out		TIMSK,TEMP_A
		sei		
		rjmp	key_decode_end
;
key_decode_no_l:
		sbrs	FLAGS,FLAG_C			; Clock?
		rjmp	key_decode_end			; nein
;
; Clock aus, Spannung ein
		cli
		in		TEMP_A,TIMSK
		cbr		TEMP_A,(1<<TOIE0)		; Timer 0 Overflow-IRQ ausschalten
		out		TIMSK,TEMP_A
		sei
;
		cbi		BER_PORT,CLK_OUT		; Clock-Out wieder auf L
		cbr		FLAGS,(1<<FLAG_C)		; Logik aus
		sbr		FLAGS,(1<<FLAG_U)		; Spannung ein
		sbi		BER_DDR,BER_RC			; RC auf GND
		rcall	set_10v
;
key_decode_end:
		ldi		TEMP_B,KEY_WAIT
;
		cbr		FLAGS_IRQ,(1<<FLAG_KEY)
		cbr		FLAGS,(1<<FLAG_IRQ)
;
		pop		TEMP_B
		pop		TEMP_A
		ret
;
;*********************************************************************
; Set 10V positiv
; Scratch-Reg: -
;*********************************************************************
set_10v:cbi		BER_DDR,BER_30V				; 30V hochohmig schalten
		clr		DIG_DP
		sbr		DIG_DP,(1<<DIG_3)
		ret
;
;*********************************************************************
; Set 30V positiv
; Scratch-Reg: -
;*********************************************************************
set_30v:sbi		BER_DDR,BER_30V				; R an GND
		clr		DIG_DP
		sbr		DIG_DP,(1<<DIG_2)
		ret
;
;*********************************************************************
; Holt die Spannung als Durchschnitt von 64 ADC-Werten und berechnet tatschlichen Spannungswert
; Out   : TEMP_B (H), TEMP_A (L)
; Scratch-Reg: TEMP_C, TEMP_D, XH, XL
;*********************************************************************
get_adc_64:
		clr		XH
		clr		XL
		ldi		TEMP_C,64				; 8*8
;
get_adc_64_wait:
		sbrs	FLAGS,FLAG_IRQ			; auf IRQ warten
		rjmp	get_adc_64_wait
;		
		add		XL,ADC_WERT_L
		adc		XH,ADC_WERT_H
		cbr		FLAGS,(1<<FLAG_IRQ)		; Flag lschen
		dec		TEMP_C
		brne	get_adc_64_wait
;
		mov		TEMP_A,XL				; Dividend
		mov		TEMP_B,XH
		clr		TEMP_C
;				
		sbis	BER_DDR,BER_30V			; +30V
		rjmp	get_adc_64_no_30v
;
		load_p	X,(3*8*8)+OFFSET_30V			
		rjmp	get_adc_64_end
;
get_adc_64_no_30v:
		load_p	X,(8*8)+OFFSET_10V
;
get_adc_64_end:
		mov		DIV_L,XL
		mov		DIV_M,XH				; Divisor
		clr		DIV_H
		rcall	div24u					; Ergebmis in TEMP_C..TEMP_A
		ret
;
;*********************************************************************
; 16 Bit Binr zu Dezimal (7-Segment) Dekoder fr 4 Digits
; IN:			TEMP_A (L), TEMP_B (H)
; OUT:			Segmentmuster in Anzeigeregister DIGIT_1...4
; Scratch-Reg:	TEMP_C
;*********************************************************************
show_16bit_wert:
		ldi		TEMP_C, -1			; fr nachfolgende Subtraktion initialisieren
Loop1000:							; 1000er-Stelle (DIGIT_4)
		inc		TEMP_C				; TEMP_C zhlt Subtraktionen bis zum negativ berlauf
		subi	TEMP_A,LOW(1000)
		sbci	TEMP_B,HIGH(1000)
		brcc	LOOP1000
		rcall	encode_seg			; Segmentmuster aus TEMP_C nach TEMP_C
		sbrc	FLAGS,FLAG_U		; bei Spannung nur 3 Stellen anzeigen
		clr		TEMP_C				; dann alle Segmente aus
		sts		DIGIT_4, TEMP_C		; => DIGIT_4
;
		ldi		TEMP_C, 10			; fr nachfolgende Addition initialisieren
Loop100:							; 100er-Stelle (DIGIT_3)
		dec		TEMP_C				; TEMP_C zhlt Additionen bis Wert wieder positiv ist
		subi	TEMP_A,LOW(-100)
		sbci	TEMP_B,HIGH(-100)
		brcs	Loop100
		rcall	encode_seg			; Segmentmuster aus TEMP_C nach TEMP_C
		sts 	DIGIT_3, TEMP_C		; => DIGIT_3
;
		ldi		TEMP_C, -1			; fr nachfolgende Subtraktion initialisieren
Loop10:								; 10er-Stelle (DIGIT_2)
		inc		TEMP_C				; TEMP_C zhlt Subtraktionen bis zum negativ berlauf
		subi	TEMP_A,10			; ab hier nur noch 1 Byte
		brcc	Loop10
		rcall	encode_seg			; Segmentmuster aus TEMP_C nach TEMP_C
		sts		DIGIT_2, TEMP_C		; => DIGIT_2
;
		ldi		TEMP_C, 10			; der Rest in TEMP_A bestimmt die 1er-Stelle (DIGIT_1)
		add		TEMP_C, TEMP_A
		rcall	encode_seg			; Segmentmuster aus TEMP_C nach TEMP_C
		sts		DIGIT_1, TEMP_C		; => DIGIT_1
		ret
;
;*********************************************************************
; Encode_seg holt die Segmentmuster fr 0-9 aus der Tabelle seg_tab
; IN:			TEMP_C mit Dezimalzahl
; OUT:			TEMP_C Segmentmuster
; Scratch-Reg:	ZH, ZL
;*********************************************************************
encode_seg:							; Die Sonderzeichen "C", "L", "P"
		load_p	Z,(2*seg_tab)		; etc werden direkt im Programm
		add		ZL,TEMP_C			; als Segmentmuster zugewiesen.
		adc		ZH,NULL
		lpm		TEMP_C,Z
		ret
;
seg_tab:;	  GFxEDCBA	Der Dezimalpunkt wird separat angesteuert.
		.db 0b01011111, 0b00000110	; 0 1
		.db 0b10011011, 0b10001111	; 2 3	Bit5 "x" = T1 Zhleingang
		.db 0b11000110, 0b11001101	; 4 5
		.db 0b11011101, 0b00000111	; 6 7	Bit gesetzt =>
		.db 0b11011111, 0b11001111	; 8 9	Segment leuchtet
;
;*********************************************************************
; Wartet rund 10ms (10 IRQ-Aufrufe)
; Scratch-Reg: -
;*********************************************************************
wait_5ms:
		push	TEMP_A
		ldi		TEMP_A,5
		cbr		FLAGS,(1<<FLAG_IRQ)
;
wait_5ms_loop:
		sbrs	FLAGS,FLAG_IRQ
		rjmp	wait_5ms_loop
;
		cbr		FLAGS,(1<<FLAG_IRQ)
		dec		TEMP_A
		brne	wait_5ms_loop
;		
		pop		TEMP_A
		ret
;
;*********************************************************************
; Init mit Ub und berechnet H_WERT
; Scratch-Reg: -
;*********************************************************************
init:	load_p	X,L_WERT_DEF				; * 10mV
		sts		L_WERT,XL
		sts		L_WERT+1,XH
;
		ldi		TEMP_A,(1<<REFS0)|(1<<MUX3)|(1<<MUX2)|(1<<MUX1)	; AVcc als Referenz, Bandgap als In (1,23V)
		out		ADMUX,TEMP_A
;
		ldi		TEMP_A,2					; 2ms warten
		cbr		FLAGS,(1<<FLAG_IRQ)
;
init_loop_1:
		sbrs	FLAGS,FLAG_IRQ
		rjmp	init_loop_1
;
		cbr		FLAGS,(1<<FLAG_IRQ)
		dec		TEMP_A
		brne	init_loop_1
;
		clr		DIV_L						; Ziel fr Messerergebnis
		clr		DIV_M
		clr		DIV_H
;
		ldi		TEMP_A,8					; 8 Messungen
;
init_loop_mess:
		sbrs	FLAGS,FLAG_IRQ
		rjmp	init_loop_mess
;
		add		DIV_L,ADC_WERT_L
		adc		DIV_M,ADC_WERT_H
		clr		DIV_H
;
		cbr		FLAGS,(1<<FLAG_IRQ)
		dec		TEMP_A
		brne	init_loop_mess				; Summe 8 Messungen in DIV_H, DIV_L
;
		ldi		TEMP_A,low(101600)			; 1,23V Bandgap schon fr die Rechnung mit 8 Messwerten normalisiert
		ldi		TEMP_B,high(101600)
		ldi		TEMP_C,byte3(101600)
		rcall	div24u						; durch Mewert teilen, Ergebnis: TEMP_C..TEMP_A
;
		ldi		TEMP_B,H_WERT_DEF			; % von Ub
		subi	TEMP_B,-3					; 300mV fr Verpolschutzdiode dazu
;
		mul		TEMP_A,TEMP_B				; Ergebnis in MUL_H, MUL_L
;
		clr		DIV_H						; /10
		clr		DIV_M
		ldi		TEMP_A,10
		mov		DIV_L,TEMP_A
;
		mov		TEMP_A,MUL_L				; Dividend fr 24/24 Division
		mov		TEMP_B,MUL_H
		clr		TEMP_C
		rcall	div24u
;
		sts		H_WERT,TEMP_A				; Ergebnis in TEMP_C..TEMP_A
		sts		H_WERT+1,TEMP_B
;
		ldi		TEMP_A,(1<<REFS0)|(1<<REFS1); interne Referenz
		out		ADMUX,TEMP_A
;
		ret
;
;******************************************************************
;* Division 24/24
;*			Dividend = TEMP_C .. TEMP_A
;* Result = ---------------------------- => TEMP_C .. TEMP_A
;*			Divisor  = DIV_H, M, L
;******************************************************************
div24u:	clr REM_L			; clear remainder Low byte
		clr REM_M			; clear remainder mid byte
		sub REM_H,REM_H		; clear remainder High byte and carry
		ldi TEMP_D,25 		; init loop counter
;
d24u_1:	rol TEMP_A			; shift left dividend
		rol TEMP_B
		rol TEMP_C
		dec TEMP_D			; decrement counter
		brne d24u_2 		; if done
		ret 				; return
;
d24u_2: rol REM_L 			; shift dividend into remainder
		rol REM_M
		rol REM_H
		sub REM_L,DIV_L		; remainder = remainder - divisor
		sbc REM_M,DIV_M
		sbc REM_H,DIV_H
		brcc d24u_3			; if result negative
		add REM_L,DIV_L		; restore remainder
		adc REM_M,DIV_M
		adc REM_H,DIV_H
		clc 				; clear carry to be shifted into result
		rjmp d24u_1 		; else
d24u_3: sec 				; set carry to be shifted into result
		rjmp d24u_1
		ret
