; Definitionen fr Mega8 Belegungen der Atmel-Ports:
;*****************************************************************
;	PB0		ICP1		Periodendauer Eingang
;   PB1		OC1A		Segment DP
;   PB2		SS			Digit 1     
;   PB3		MOSI		Digit 2      
;   PB4		MISO		Digit 3
;   PB5		MSCK		Digit 4
;	PB6		XTAL1
;	PB7		XTAL2
				
;   PC0		ADC0		Spannung Eingang
;   PC1		ADC1		Clock Ausgang
;   PC2		ADC2		KEY	
;   PC3		ADC3		R IN				1M vom Eingang
;	PC4		I2C SDA		68k/10n				bei Spannung gegen GND schalten
;	PC5		I2C SCL		27k					bei 30V gegen GND schalten
;	PC6		RESET

;   PD0		RxD			Segment A
;   PD1		TxD			Segment B
;   PD2		INT0		Segment C
;   PD3		INT1		Segment D
;   PD4	    T0			Segment E
;   PD5		T1			Frequenz Eingang
;   PD6		AIN0		Segment F
;   PD7		AIN1		Segment G
;*****************************************************************

;--- Allgemeines -----------------------------------------------

.equ    F_CPU           = 16000000

.equ	IRQ_TIME		= 1					; IRQ-Zeit in ms

.equ	VORTEILER		= 64
.equ	COUNT_IN		= F_CPU/VORTEILER	; Vorteiler 64 -> 250000Hz

.equ    TCONST_TIME     = (COUNT_IN/(1000/IRQ_TIME))-1 ; 249

.equ	KEY_WAIT		= 30				; ms Tasten-Entprellzeit

.equ	OFFSET_10V		= -6				; Korrekturfaktor Referenzspannung und Widerstnde
.equ	OFFSET_30V		=  6				; Korrekturfaktor Referenzspannung und Widerstnde

.equ	L_WERT_DEF		= 20				; * 10mV als Obergrenze L-Pegel
.equ	H_WERT_DEF		= 60				; in % von Ub als Untergrenze H-Pegel

;**************************** Port-Bits ***************************

.equ	SEG_PORT		= PORTD
.equ	SEG_A			= PD0
.equ	SEG_B			= PD1
.equ	SEG_C			= PD2
.equ	SEG_D			= PD3
.equ	SEG_E			= PD4

.equ	SEG_F			= PD6
.equ	SEG_G			= PD7

.equ	DIG_PORT		= PORTB

.equ	LOGIK_PORT		= PINB
.equ	LOGIK_PIN		= PB0				; nutzt IRQ-Interrupt

.equ	SEG_DP			= PB1

.equ	DIG_1			= PB2
.equ	DIG_2			= PB3
.equ	DIG_3			= PB4
.equ	DIG_4			= PB5

.equ	KEY_PORT		= PINC
.equ	KEY_PIN			= PC2

.equ	CLK_OUT			= PC1				; Ausgang auf L in Grundstellung

.equ	BER_DDR			= DDRC				; Port fr Spannungsteiler, Umschaltung auf hochohmig
.equ	BER_PORT		= PORTC				; Port fr Spannungsteiler, Umschaltung GND / Ub

.equ	BER_R_IN		= PC3				; 1M vom Eingang

.equ	BER_RC			= PC4				; C und 10V-Teilerwiderstand
.equ	BER_30V			= PC5				; 30V-Teilerwiderstand

;*************************** FLAGS IRQ *****************************

.equ	FLAG_KEY		= 0					; Taste als gedrckt erkannt 
.equ	FLAG_KEY_PRESS	= 1					; Taste noch nicht losgelassen

.equ	FLAG_FREQ		= 2					; Frequenzmessung gestartet
.equ	FLAG_PERIODE	= 3					; Periodendauermessung gestartet
.equ	FLAG_PERIODE_1F	= 4					; Periodendauermessung warten auf 1. Signalflanke

;***************************** FLAGS ******************************

.equ	FLAG_IRQ		= 0					; 1ms IRQ wurde ausgefhrt

.equ	FLAG_U			= 1					; Spannung
.equ	FLAG_F			= 2					; Frequenz
.equ	FLAG_L			= 3					; Logik
.equ	FLAG_C			= 4					; Clock Out

;**************************** Ram-Reservierungen ******************
;
;--- Internes RAM ------------------------------------------------
;
.DSEG
.ORG		SRAM_START						; $060 bei ATmega8
;
DIGIT_1:	.byte 1							; rechtes Digit (Einer)
DIGIT_2:	.byte 1
DIGIT_3:	.byte 1
DIGIT_4:	.byte 1							; linkes Digit (Tausender)
;
L_WERT:		.byte 2							; Wert * 10mV Schwellwerte fr Logiktest
H_WERT:		.byte 2							; Wert * 10mV
;
;**************************** Register ****************************
;
;--- lower Register -----------------------------------------------
;
.def	MUL_L		= r0
.def	MUL_H		= r1
;
.def	NULL		= r2
.def	SREG_SAVE	= r3
;
.def	ADC_WERT_L	= r4
.def	ADC_WERT_H	= r5
;
.def	CNT_WERT_L	= r6
.def	CNT_WERT_H	= r7
;
.def	KEY_TEMP	= r8					; Temp fr Taste
;
.def	DIV_L		= r9					; Divisor L 24/24 Bit-Division
.def	DIV_M		= r10					; Divisor M 24/24 Bit-Division
.def	DIV_H		= r11					; Divisor H 24/24 Bit-Division
;
.def	REM_L		= r12					; Remainder L 24/24 Bit-Division
.def	REM_M		= r13					; Remainder M 24/24 Bit-Division
.def	REM_H		= r14					; Remainder H 24/24 Bit-Division
;
.def	Ofl_T1		= r15					; zhlt Timer1-berlufe (bei Periodendauermessung)
;
;--- higher Register ----------------------------------------------
;
.def    TEMP_A      = r16                   ; Temp-Register, auch Dividend & Ergebnis L 24/24 Bit-Division
.def    TEMP_B      = r17                   ; Temp-Register, auch Dividend & Ergebnis M 24/24 Bit-Division
.def	TEMP_C		= r18                   ; Temp-Register, auch Dividend & Ergebnis H 24/24 Bit-Division
.def	TEMP_D		= r19					; Temp-Register

.def	FLAGS		= r20					; Flags fr Funktionen
.def	FLAGS_IRQ	= r21					; Flags der IRQ-Routinen

.def	KEY_CNT		= r22					; Taste Zhler entprellen

.def	DIG_NR		= r23					; Bit auf 1 fr Digit-Position
.def	DIG_ADR		= r24					; Low-Byte der Digit-Adresse im Ram
.def	DIG_DP		= r25					; Dezimalpunkt, 1 an Digitposition -> DP an

; r26/r27, r30/r31 von X, Z benutzt, r28/r29 fr Periodendauer-Messung

;--- Macros -------------------------------------------------------

.macro  load_p                      ; ldt 16Bit @1 nach @0 High und @0 Low
        ldi     @0H,HIGH(@1)
        ldi     @0L,LOW(@1)
.endmacro
